@@ -229,14 +229,14 @@ class ByoGs:
             if platform == "win32":
                 response = subprocess.run([
                     # f"docker build --no-cache -t {tag} {path}"
-                    'docker', 'build', '--platform', build_platform, '-t', self.tag,
+                    'docker', 'build', '--load', '--platform', build_platform, '-t', self.tag,
                     '-f', self.docker_path_filename,  self.path
                 ], shell=True, check=False, env=env)
                 # stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
             else:
                 response = subprocess.run([
                     # f"docker build --no-cache -t {tag} {path}"
-                    "docker build --platform " +
+                    "docker build --load --platform " +
                     f"{build_platform} -t {self.tag} " +
                     f"-f {self.docker_path_filename} {self.path}"
                 ], shell=True, check=False, env=env)
@@ -400,8 +400,9 @@ class ByoGs:
                     code=SNAPCTL_INPUT_ERROR)
             # Check docker file path
             if not self.skip_build and not self.docker_path_filename:
+                path_in_message = self.resources_path if self.resources_path else self.path
                 snapctl_error(
-                    f"Unable to find {self.docker_path_filename}", SNAPCTL_INPUT_ERROR)
+                    f"Unable to find the Dockerfile at {path_in_message}", SNAPCTL_INPUT_ERROR)
         elif self.subcommand == 'sync':
             # Check path
             if not self.skip_build and not self.path:
