@@ -748,14 +748,14 @@ class ByoSnap:
             if platform == "win32":
                 response = subprocess.run([
                     # f"docker build --no-cache -t {remote_tag} {path}"
-                    'docker', 'build', '--platform', build_platform, '-t', self.remote_tag,
+                    'docker', 'build', '--load', '--platform', build_platform, '-t', self.remote_tag,
                     '-f', self.docker_path_filename,  self.path
                 ], shell=True, check=False, env=env)
                 # stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
             else:
                 response = subprocess.run([
                     # f"docker build --no-cache -t {remote_tag} {path}"
-                    "docker build --platform " +
+                    "docker build --load --platform " +
                     f"{build_platform} -t {self.remote_tag} " +
                     f"-f {self.docker_path_filename} {self.path}"
                 ], shell=True, check=False, env=env)
@@ -1350,7 +1350,7 @@ class ByoSnap:
         if self.path is not None or self.resources_path is not None:
             self.upload_docs(no_exit=True)
         snapctl_success(
-            message='BYOSNAP publish successful', no_exit=no_exit)
+            message='BYOSNAP publish image successful', no_exit=no_exit)
 
     def publish_version(self, no_exit: bool = False) -> None:
         """
