@@ -20,6 +20,7 @@ from snapctl.config.constants import SERVER_CALL_TIMEOUT, SNAPCTL_INPUT_ERROR, \
 from snapctl.utils.helper import snapctl_error, snapctl_success, get_dot_snapser_dir
 from snapctl.utils.echo import info, warning
 
+
 class Byows:
     """
       CLI commands exposed for a Bring your own Workstation
@@ -106,7 +107,7 @@ class Byows:
         env_vars = []
 
         for snap_id in snap_ids:
-            upper_id = snap_id.upper()
+            upper_id = snap_id.upper().replace("-", "_")
             env_vars.append(f"SNAPEND_{upper_id}_GRPC_URL=localhost:{port}")
             env_vars.append(
                 f"SNAPEND_{upper_id}_HTTP_URL=http://localhost:{port}")
@@ -128,7 +129,7 @@ class Byows:
         env_lines = []
 
         for snap_id in snap_ids:
-            upper_id = snap_id.upper()
+            upper_id = snap_id.upper().replace("-", "_")
             env_lines.append(
                 f"export SNAPEND_{upper_id}_GRPC_URL=localhost:{port}")
             env_lines.append(
@@ -393,7 +394,7 @@ class Byows:
                     self._handle_signal))
                 if hasattr(signal, "SIGBREAK"):
                     signal.signal(signal.SIGBREAK,
-                    functools.partial(self._handle_signal))
+                                  functools.partial(self._handle_signal))
                 # Set up port forward
                 self._setup_port_forward(
                     response_json['proxyPrivateKey'],
@@ -416,7 +417,8 @@ class Byows:
                 code=SNAPCTL_BYOWS_ATTACH_ERROR, progress=progress)
         except HTTPError as http_err:
             snapctl_error(
-                message=Byows._format_portal_http_error("Attach failed.", http_err, res),
+                message=Byows._format_portal_http_error(
+                    "Attach failed.", http_err, res),
                 code=SNAPCTL_BYOWS_ATTACH_ERROR, progress=progress)
         except RequestException as e:
             snapctl_error(
