@@ -18,12 +18,16 @@ from snapctl.config.constants import SERVER_CALL_TIMEOUT, SNAPCTL_INPUT_ERROR, \
     SNAPCTL_SNAPEND_APPLY_ERROR, SNAPCTL_SNAPEND_PROMOTE_SERVER_ERROR, \
     SNAPCTL_SNAPEND_PROMOTE_TIMEOUT_ERROR, SNAPCTL_SNAPEND_PROMOTE_ERROR, \
     SNAPCTL_SNAPEND_DOWNLOAD_ERROR, SNAPCTL_SNAPEND_UPDATE_TIMEOUT_ERROR, \
-    SNAPCTL_SNAPEND_UPDATE_ERROR, SNAPCTL_SNAPEND_STATE_ERROR, HTTP_ERROR_SNAPEND_MANIFEST_MISMATCH, \
-    SNAPCTL_SNAPEND_APPLY_MANIFEST_MISMATCH_ERROR
+    SNAPCTL_SNAPEND_UPDATE_ERROR, SNAPCTL_SNAPEND_STATE_ERROR, \
+    HTTP_ERROR_SNAPEND_MANIFEST_MISMATCH, HTTP_ERROR_CLUSTER_UPDATE_IN_PROGRESS, \
+    SNAPCTL_SNAPEND_APPLY_MANIFEST_MISMATCH_ERROR, HTTP_ERROR_GAME_NOT_FOUND, \
+    SNAPCTL_SNAPEND_CREATE_SERVER_ERROR, SNAPCTL_SNAPEND_CREATE_TIMEOUT_ERROR, \
+    SNAPCTL_SNAPEND_CREATE_ERROR, HTTP_CONFLICT
 from snapctl.config.hashes import PROTOS_TYPES, CLIENT_SDK_TYPES, SERVER_SDK_TYPES, \
     SNAPEND_MANIFEST_TYPES, SDK_TYPES, SDK_ACCESS_AUTH_TYPE_LOOKUP
 from snapctl.utils.echo import error, success, info
 from snapctl.utils.helper import snapctl_error, snapctl_success
+from snapctl.utils.exceptions import SnapendDownloadException
 
 
 class Snapend:
@@ -31,11 +35,11 @@ class Snapend:
       CLI commands exposed for a Snapend
     """
     SUBCOMMANDS = [
-        'enumerate', 'clone', 'apply',
+        'enumerate', 'create', 'clone', 'apply',
         'download', 'update', 'state'
     ]
     DOWNLOAD_CATEGORY = [
-        'sdk', 'protos', 'snapend-manifest'
+        'sdk', 'protos', 'snapend-manifest', 'legacy-sdk'
     ]
     CATEGORY_TYPE_SDK = [
         # 'omni',
@@ -79,8 +83,8 @@ class Snapend:
         self.api_key: str = api_key
         self.snapend_id: str = snapend_id
         self.game_id: Union[str, None] = game_id
-        self.name: str = name
-        self.env: str = env
+        self.name: Union[str, None] = name
+        self.env: Union[str, None] = env
         self.manifest_path_filename: Union[str, None] = manifest_path_filename
         self.force: bool = force
         self.category: str = category
@@ -103,6 +107,9 @@ class Snapend:
         self.byogs_list: Union[str, None] = Snapend._make_byogs_list(
             byogs) if byogs else None
         self.blocking: bool = blocking
+        # Backup variables
+        self.manifest_name: Union[str, None] = None
+        self.manifest_environment: Union[str, None] = None
         # Validate input
         self.validate_input()
 
@@ -123,6 +130,10 @@ class Snapend:
             The portal server however expects us to pass client-sdk or server-sdk
             Hence we need to do this
         '''
+        if category == 'legacy-sdk' and category_format in CLIENT_SDK_TYPES:
+            return 'legacy-client-sdk'
+        if category == 'legacy-sdk' and category_format in SERVER_SDK_TYPES:
+            return 'legacy-server-sdk'
         if category == 'sdk' and category_format in CLIENT_SDK_TYPES:
             return 'client-sdk'
         if category == 'sdk' and category_format in SERVER_SDK_TYPES:
@@ -135,7 +146,7 @@ class Snapend:
 
     @staticmethod
     def _make_download_type(category: str):
-        if category == 'sdk':
+        if category in ['sdk', 'legacy-sdk']:
             return SDK_TYPES
         if category == 'protos':
             return PROTOS_TYPES
@@ -219,6 +230,7 @@ class Snapend:
 
     def _execute_download(self) -> bool:
         try:
+            final_http_lib = None
             url = (
                 f"{self.base_url}/v1/snapser-api/snapends/{self.snapend_id}/"
                 f"download?category={self.portal_category}"
@@ -230,23 +242,47 @@ class Snapend:
             if self.category == 'protos':
                 url += f"&subtype={self.category_type}"
             # If client or server SDK, add sub type and auth type
-            if self.category == 'sdk':
+            if self.category in ['sdk', 'legacy-sdk']:
                 url += "&subtype=" + \
                     f"{self.download_types[self.category_format]['subtype']}"
                 url += f"&access_type={SDK_ACCESS_AUTH_TYPE_LOOKUP[self.category_type]['access_type']}"
                 if 'auth_type' in SDK_ACCESS_AUTH_TYPE_LOOKUP[self.category_type] and \
                         SDK_ACCESS_AUTH_TYPE_LOOKUP[self.category_type]['auth_type'] != '':
                     url += f"&auth_type={SDK_ACCESS_AUTH_TYPE_LOOKUP[self.category_type]['auth_type']}"
-                if self.category_http_lib:
-                    # First check if the format supports http-lib
-                    if self.category_format in Snapend.get_formats_supporting_http_lib():
-                        url += f"&http_lib={self.category_http_lib}"
+                if self.category_format in Snapend.get_formats_supporting_http_lib():
+                    http_libs = Snapend.get_http_lib_for_sdk(
+                        self.category_format
+                    )
+                    if self.category_http_lib:
+                        if self.category_http_lib in http_libs:
+                            final_http_lib = self.category_http_lib
+                    else:
+                        if len(http_libs) > 0:
+                            info(f"Using default `--http-lib {http_libs[0]}`")
+                            final_http_lib = http_libs[0]
+                    if final_http_lib:
+                        url += f"&http_lib={final_http_lib}"
             # Customize snaps
             if self.snaps:
                 url += f"&snaps={self.snaps}"
+            # info(f"Downloading from {url}")
             res = requests.get(
                 url, headers={'api-key': self.api_key}, timeout=SERVER_CALL_TIMEOUT
             )
+            if not res.ok:
+                # Handle known conflict case
+                response = res.json()
+                if res.status_code == HTTP_CONFLICT:
+                    if (
+                        response.get(
+                            "api_error_code") == HTTP_ERROR_CLUSTER_UPDATE_IN_PROGRESS
+                    ):
+                        raise SnapendDownloadException(
+                            "Snapend update is in progress. Please try again later.")
+                raise SnapendDownloadException(
+                    f"Unable to download {self.category}. " +
+                    f"Reason: {response.get('message', '')}"
+                )
             fn: str = ''
             if self.category == 'admin-settings':
                 fn = f"snapser-{self.snapend_id}-admin-settings.json"
@@ -265,22 +301,21 @@ class Snapend:
                     fn += f"-{SDK_ACCESS_AUTH_TYPE_LOOKUP[self.category_type]['auth_type']}"
                 if self.category_http_lib:
                     # First check if the format supports http-lib
-                    if self.category_format in Snapend.get_formats_supporting_http_lib():
-                        fn += f"-{self.category_http_lib}"
+                    if final_http_lib:
+                        fn += f"-{final_http_lib}"
                 fn += ".zip"
             if self.out_path is not None:
                 file_save_path = os.path.join(self.out_path, fn)
             else:
                 file_save_path = os.path.join(os.getcwd(), fn)
-            if res.ok:
-                content: bytes = res.content
-                with open(file_save_path, "wb") as file:
-                    if self.category in ['admin-settings']:
-                        content = json.loads(res.content)
-                        json.dump(content, file, indent=4)
-                    else:
-                        file.write(res.content)
-                return True
+
+            with open(file_save_path, "wb") as file:
+                if self.category in ['admin-settings']:
+                    content = json.loads(res.content)
+                    json.dump(content, file, indent=4)
+                else:
+                    file.write(res.content)
+            return True
         except RequestException as e:
             info(
                 f"Exception: Unable to download {self.category}. Reason: {e}"
@@ -298,6 +333,15 @@ class Snapend:
                 format_list.append(key)
         return format_list
 
+    @staticmethod
+    def get_http_lib_for_sdk(sdk_format: str) -> list:
+        '''
+            Get the list of http-lib supported for a sdk format
+        '''
+        if sdk_format in SDK_TYPES and 'http-lib' in SDK_TYPES[sdk_format]:
+            return SDK_TYPES[sdk_format]['http-lib']
+        return []
+
     @staticmethod
     def get_http_formats_str() -> str:
         '''
@@ -311,6 +355,59 @@ class Snapend:
             format_str = format_str[:-2]
         return format_str
 
+    # Backup variables
+
+    def setup_manifest_name_and_env_vars(self) -> bool:
+        """
+        Read a manifest (JSON or YAML) and saves the name and environment.
+        Supports extensions: .json, .yaml, .yml
+        If the extension is unknown, tries JSON then YAML.
+        """
+        def parse_json(s: str):
+            return json.loads(s)
+
+        def parse_yaml(s: str):
+            try:
+                import yaml  # type: ignore
+            except ImportError as e:
+                raise RuntimeError(
+                    "YAML file provided but PyYAML is not installed. "
+                    "Install with: pip install pyyaml"
+                ) from e
+            return yaml.safe_load(s)
+
+        with open(self.manifest_path_filename, "r", encoding="utf-8") as f:
+            text = f.read()
+
+        ext = os.path.splitext(self.manifest_path_filename)[1].lower()
+        if ext == ".json":
+            parsers = (parse_json, parse_yaml)
+        elif ext in (".yaml", ".yml"):
+            parsers = (parse_yaml, parse_json)
+        else:
+            parsers = (parse_json, parse_yaml)
+
+        last_err = None
+        data = None
+        for parser in parsers:
+            try:
+                data = parser(text)
+                break
+            except Exception as e:
+                last_err = e
+
+        if data is None:
+            return False
+        if not isinstance(data, dict):
+            return False
+
+        try:
+            self.manifest_name = str(data["name"])
+            self.manifest_environment = str(data["environment"])
+        except KeyError as e:
+            pass
+        return False
+
     # Validate input
     def validate_input(self) -> None:
         """
@@ -330,26 +427,13 @@ class Snapend:
         if self.subcommand == 'enumerate':
             if not self.game_id:
                 snapctl_error(
-                    message="Missing required parameter: game_id",
+                    message="Missing required parameter: application-id",
                     code=SNAPCTL_INPUT_ERROR)
-        elif self.subcommand == 'clone':
+        elif self.subcommand == 'create':
             if not self.game_id:
                 snapctl_error(
-                    message="Missing required parameter: game_id",
+                    message="Missing required parameter: application-id",
                     code=SNAPCTL_INPUT_ERROR)
-            if not self.name:
-                snapctl_error(
-                    message="Missing required parameter: name",
-                    code=SNAPCTL_INPUT_ERROR)
-            if not self.env:
-                snapctl_error(
-                    message="Missing required parameter: env", code=SNAPCTL_INPUT_ERROR)
-            if self.env.upper() not in Snapend.ENV_TYPES:
-                snapctl_error(
-                    message="Invalid environment. Valid environments are " +
-                    f"{', '.join(Snapend.ENV_TYPES)}.",
-                    code=SNAPCTL_INPUT_ERROR
-                )
             if not self.manifest_path_filename:
                 snapctl_error(
                     message="Missing required parameter: manifest_path_filename",
@@ -360,6 +444,67 @@ class Snapend:
                     "Please enter a valid path to the manifest file",
                     code=SNAPCTL_INPUT_ERROR
                 )
+            if not self.manifest_file_name:
+                snapctl_error(
+                    message="Invalid manifest file. Supported formats are .json, .yml, .yaml",
+                    code=SNAPCTL_INPUT_ERROR)
+            self.setup_manifest_name_and_env_vars()
+            if not self.name and not self.manifest_name:
+                snapctl_error(
+                    message="Missing name parameter or name in manifest is required for " +
+                    "create command.",
+                    code=SNAPCTL_INPUT_ERROR)
+            if not self.env and not self.manifest_environment:
+                snapctl_error(
+                    message="Missing environment parameter or environment in manifest is " +
+                    "required for create command.",
+                    code=SNAPCTL_INPUT_ERROR)
+            if self.env and self.env.upper() not in Snapend.ENV_TYPES and \
+                    self.manifest_environment not in Snapend.ENV_TYPES:
+                env_str = 'Invalid environment argument.' if not self.env else 'Invalid environment in manifest.'
+                snapctl_error(
+                    message=f"{env_str}. Valid environments are " +
+                    f"{', '.join(Snapend.ENV_TYPES)}.",
+                    code=SNAPCTL_INPUT_ERROR
+                )
+        elif self.subcommand == 'clone':
+            if not self.game_id:
+                snapctl_error(
+                    message="Missing required parameter: application-id",
+                    code=SNAPCTL_INPUT_ERROR)
+            if not self.manifest_path_filename:
+                snapctl_error(
+                    message="Missing required parameter: manifest_path_filename",
+                    code=SNAPCTL_INPUT_ERROR)
+            if not os.path.isfile(self.manifest_path_filename):
+                snapctl_error(
+                    message=f"Invalid path {self.manifest_path_filename}. " +
+                    "Please enter a valid path to the manifest file",
+                    code=SNAPCTL_INPUT_ERROR
+                )
+            if not self.manifest_file_name:
+                snapctl_error(
+                    message="Invalid manifest file. Supported formats are .json, .yml, .yaml",
+                    code=SNAPCTL_INPUT_ERROR)
+            self.setup_manifest_name_and_env_vars()
+            if not self.name and not self.manifest_name:
+                snapctl_error(
+                    message="Missing name parameter or name in manifest is required for " +
+                    "create command.",
+                    code=SNAPCTL_INPUT_ERROR)
+            if not self.env and not self.manifest_environment:
+                snapctl_error(
+                    message="Missing environment parameter or environment in manifest is " +
+                    "required for create command.",
+                    code=SNAPCTL_INPUT_ERROR)
+            if self.env and self.env.upper() not in Snapend.ENV_TYPES and \
+                    self.manifest_environment not in Snapend.ENV_TYPES:
+                env_str = 'Invalid environment argument.' if not self.env else 'Invalid environment in manifest.'
+                snapctl_error(
+                    message=f"{env_str}. Valid environments are " +
+                    f"{', '.join(Snapend.ENV_TYPES)}.",
+                    code=SNAPCTL_INPUT_ERROR
+                )
         elif self.subcommand == 'apply':
             if not self.manifest_path_filename:
                 snapctl_error(
@@ -405,7 +550,7 @@ class Snapend:
                     code=SNAPCTL_INPUT_ERROR
                 )
             # Check the auth type
-            if self.category == 'sdk':
+            if self.category in ['sdk', 'legacy-sdk']:
                 if not self.category_type:
                     snapctl_error(
                         message="Missing required parameter: --type",
@@ -433,10 +578,12 @@ class Snapend:
                     #         code=SNAPCTL_INPUT_ERROR
                     #     )
                 if self.category_http_lib:
-                    # First check if the format supports http-lib
+                    # First check if the format supports http-lib TODO
                     if self.category_format in Snapend.get_formats_supporting_http_lib():
                         # Check if the http-lib is supported for the format
-                        valid_http_libs = SDK_TYPES[self.category_format]['http-lib']
+                        valid_http_libs = Snapend.get_http_lib_for_sdk(
+                            self.category_format
+                        )
                         if self.category_http_lib not in valid_http_libs:
                             snapctl_error(
                                 message="Invalid HTTP Library. Valid libraries are " +
@@ -511,7 +658,7 @@ class Snapend:
             message="Unable to enumerate snapends.",
             code=SNAPCTL_SNAPEND_ENUMERATE_ERROR, progress=progress)
 
-    def clone(self) -> None:
+    def create(self) -> None:
         """
           Create a Snapend from a manifest
         """
@@ -521,6 +668,92 @@ class Snapend:
             transient=True,
         )
         progress.start()
+        progress.add_task(
+            description='Creating your snapend from the manifest...', total=None)
+        try:
+            with open(self.manifest_path_filename, 'rb') as file:
+                files = {'snapend-manifest': file}
+                payload = {
+                    'game_id': self.game_id,
+                    'name': self.name if self.name else self.manifest_name,
+                    'env': self.env.upper() if self.env else self.manifest_environment.upper(),
+                    'ext': self.manifest_file_name.split('.')[-1]
+                }
+                url = f"{self.base_url}/v1/snapser-api/snapends/snapend-manifest"
+                res = requests.post(
+                    url, headers={'api-key': self.api_key},
+                    files=files, data=payload, timeout=SERVER_CALL_TIMEOUT
+                )
+                if not res.ok:
+                    response = res.json()
+                    if 'details' in response:
+                        response = response['details']
+                    snapctl_error(
+                        message=response,
+                        code=SNAPCTL_SNAPEND_CREATE_ERROR,
+                        progress=progress
+                    )
+                # extract the cluster ID
+                response = res.json()
+                if 'cluster' not in response or 'id' not in response['cluster']:
+                    snapctl_error(
+                        message='Server Error. Unable to get a Snapend ID. ' +
+                        'Please try again in sometime.',
+                        code=SNAPCTL_SNAPEND_CREATE_SERVER_ERROR,
+                        progress=progress
+                    )
+                self._assign_snapend_id(response['cluster']['id'])
+                info(f"Cluster ID assigned: {response['cluster']['id']}")
+                if self.blocking:
+                    snapctl_success(
+                        message='Snapend create initiated.',
+                        progress=progress,
+                        no_exit=True
+                    )
+                    status = self._blocking_get_status()
+                    # Fetch the new manifest
+                    if status is True:
+                        # TODO: Uncomment this if we want to do an auto download
+                        # self._setup_for_download(
+                        #     self.manifest_file_name.split('.')[-1])
+                        # self._execute_download()
+                        snapctl_success(
+                            message='Snapend create successful. Do not forget to download the latest manifest.',
+                            progress=progress)
+                    snapctl_error(
+                        message='Snapend create has been initiated but the Snapend is not up yet.' +
+                        'Please try checking the status of the Snapend in some time',
+                        code=SNAPCTL_SNAPEND_CREATE_TIMEOUT_ERROR,
+                        progress=progress
+                    )
+                snapctl_success(
+                    message="Snapend create has been initiated. " +
+                    "You can check the status using " +
+                    "`snapctl snapend state --snapend-id" +
+                    f"{response['cluster']['id']}`",
+                    progress=progress
+                )
+        except RequestException as e:
+            snapctl_error(
+                message=f"Unable to create a snapend from a manifest. {e}",
+                code=SNAPCTL_SNAPEND_CREATE_ERROR, progress=progress
+            )
+        finally:
+            progress.stop()
+        snapctl_error(
+            message='Unable to create a snapend from a manifest.',
+            code=SNAPCTL_SNAPEND_CREATE_ERROR, progress=progress)
+
+    def clone(self) -> None:
+        """
+          Clone a Snapend from a manifest
+        """
+        progress = Progress(
+            SpinnerColumn(),
+            TextColumn("[progress.description]{task.description}"),
+            transient=True,
+        )
+        progress.start()
         progress.add_task(
             description='Cloning your snapend from the manifest...', total=None)
         try:
@@ -528,8 +761,8 @@ class Snapend:
                 files = {'snapend-manifest': file}
                 payload = {
                     'game_id': self.game_id,
-                    'name': self.name,
-                    'env': self.env.upper(),
+                    'name': self.name if self.name else self.manifest_name,
+                    'env': self.env.upper() if self.env else self.manifest_environment.upper(),
                     'ext': self.manifest_file_name.split('.')[-1]
                 }
                 url = f"{self.base_url}/v1/snapser-api/snapends/snapend-manifest"
@@ -586,7 +819,7 @@ class Snapend:
         finally:
             progress.stop()
         snapctl_error(
-            message='Unable to apply the manifest.',
+            message='Unable to clone from the manifest.',
             code=SNAPCTL_SNAPEND_CLONE_ERROR, progress=progress)
 
     def apply(self) -> None:
@@ -658,10 +891,25 @@ class Snapend:
                 else:
                     if "api_error_code" in response and "message" in response:
                         if response['api_error_code'] == HTTP_ERROR_SNAPEND_MANIFEST_MISMATCH:
+                            msg = 'Remote manifest does not match the manifest in the ' + \
+                                'applied_configuration field.'
                             snapctl_error(
-                                message='Remote manifest does not match the manifest in the applied_configuration field.',
+                                message=msg,
                                 code=SNAPCTL_SNAPEND_APPLY_MANIFEST_MISMATCH_ERROR, progress=progress
                             )
+                        elif response['api_error_code'] == HTTP_ERROR_GAME_NOT_FOUND:
+                            msg = 'Game not found. Did you mean to use the clone command to ' + \
+                                'create a new snapend?'
+                            snapctl_error(
+                                message=msg,
+                                code=SNAPCTL_SNAPEND_APPLY_ERROR, progress=progress
+                            )
+                        elif response['api_error_code'] == HTTP_ERROR_CLUSTER_UPDATE_IN_PROGRESS:
+                            msg = 'Snapend update is in progress. Please try again later.'
+                            snapctl_error(
+                                message=msg,
+                                code=SNAPCTL_SNAPEND_APPLY_ERROR, progress=progress
+                            )
         except RequestException as e:
             snapctl_error(
                 message=f"Unable to apply the manifest snapend. {e}",
@@ -770,6 +1018,12 @@ class Snapend:
                     f"{self.category} saved at {self.out_path}",
                     progress=progress
                 )
+        except SnapendDownloadException as e:
+            snapctl_error(
+                message=str(e),
+                code=SNAPCTL_SNAPEND_DOWNLOAD_ERROR,
+                progress=progress
+            )
         except RequestException as e:
             snapctl_error(
                 message=f"Unable to download {self.category}. Reason: {e}",
