@@ -8,12 +8,15 @@ from typing import Union
 import typer
 import pyfiglet
 
+from snapctl.commands.application import Application
 from snapctl.commands.byosnap import ByoSnap
 from snapctl.commands.byogs import ByoGs
 from snapctl.commands.game import Game
 from snapctl.commands.generate import Generate
 from snapctl.commands.snapend import Snapend
 from snapctl.commands.byows import Byows
+from snapctl.commands.snaps import Snaps
+from snapctl.commands.snapend_manifest import SnapendManifest
 from snapctl.commands.release_notes import ReleaseNotes
 from snapctl.config.constants import COMPANY_NAME, API_KEY, URL_KEY, CONFIG_FILE_MAC, \
     CONFIG_FILE_WIN, DEFAULT_PROFILE, VERSION, SNAPCTL_SUCCESS, CONFIG_PATH_KEY, \
@@ -21,7 +24,7 @@ from snapctl.config.constants import COMPANY_NAME, API_KEY, URL_KEY, CONFIG_FILE
 from snapctl.config.endpoints import END_POINTS, GATEWAY_END_POINTS
 from snapctl.config.hashes import PROTOS_TYPES, SERVICE_IDS, \
     SNAPEND_MANIFEST_TYPES, SDK_TYPES
-from snapctl.utils.echo import error, success, info
+from snapctl.utils.echo import error, success, info, warning
 from snapctl.utils.helper import validate_api_key
 from snapctl.utils.telemetry import telemetry
 
@@ -259,10 +262,12 @@ def common(
 def validate(
     ctx: typer.Context,
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -282,13 +287,16 @@ def release_notes(
         ..., help="Release Notes Subcommands: " + ", ".join(ReleaseNotes.SUBCOMMANDS) + "."
     ),
     version: str = typer.Option(
-        VERSION_PREFIX + VERSION, help="(optional: show-version) If not passed will show the latest version."
+        VERSION_PREFIX + VERSION,
+        help="(optional: show-version) If not passed will show the latest version."
     ),
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -313,28 +321,37 @@ def byogs(
     ),
     tag: str = typer.Option(
         None, "--tag",
-        help="(req: build, push, publish) Tag for your snap"
+        help="(required: build, push, publish) Tag for your snap."
     ),
     # publish and publish-image
     path: Union[str, None] = typer.Option(
-        None, "--path", help="(req: build, publish) Path to your snap code"
+        None, "--path",
+        help="(required: build, publish) Path to your snap code."
     ),
     resources_path: Union[str, None] = typer.Option(
-        None, "--resources-path", help="(optional: publish) Path to resources such as your Dockerfile, swagger.json or README.md"
+        None, "--resources-path",
+        help=(
+            "(optional: publish) Path to resources such as your Dockerfile, "
+            "swagger.json or README.md."
+        )
     ),
     docker_filename: str = typer.Option(
-        "Dockerfile", help="(optional: publish) Dockerfile name to use"
+        "Dockerfile", help="(optional: publish) Dockerfile name to use."
     ),
     skip_build: bool = typer.Option(
-        False, "--skip-build", help="(optional: publish) Skip the build step. You have to pass the image tag you used during the build step."
+        False, "--skip-build",
+        help=(
+            "(optional: publish) Skip the build step. You have to pass the image tag you "
+            "used during the build step."
+        )
     ),
     snapend_id: str = typer.Option(
         None, "--snapend-id",
-        help=("(req: sync) Snapend Id.")
+        help=("(required: sync) Snapend Id.")
     ),
     fleet_names: str = typer.Option(
         None, "--fleet-names",
-        help=("(req: sync) Comma separated fleet names.")
+        help=("(required: sync) Comma separated fleet names.")
     ),
     blocking: bool = typer.Option(
         False, "--blocking",
@@ -345,10 +362,12 @@ def byogs(
     ),
     # overrides
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -379,27 +398,45 @@ def byosnap(
     ctx: typer.Context,
     # Required fields
     subcommand: str = typer.Argument(
-        ..., help="BYOSnap Subcommands: " + ", ".join(ByoSnap.SHOW_SUBCOMMANDS) + ". Commands to be deprecated soon: " + ", ".join(ByoSnap.TO_DEPRECATE_SUBCOMMANDS) + "."
+        ...,
+        help=(
+            "BYOSnap Subcommands: " +
+            ", ".join(ByoSnap.SHOW_SUBCOMMANDS) + ". Commands to be deprecated soon: " +
+            ", ".join(ByoSnap.TO_DEPRECATE_SUBCOMMANDS) + "."
+        )
     ),
     byosnap_id: Union[str, None] = typer.Option(
-        None, "--byosnap-id", help="(req: 'publish', 'sync', 'upload-docs', 'create', 'publish-image', 'publish-version', 'update-version') BYOSnap Id. Should start with byosnap-"
+        None, "--byosnap-id",
+        help=(
+            "(required: publish, sync, upload-docs, create, publish-image, publish-version, "
+            "update-version) BYOSnap Id. Should start with `byosnap-`."
+        )
     ),
     # publish
     path: Union[str, None] = typer.Option(
-        None, "--path", help="(req: publish, sync, publish-image, publish-version) Path to your snap code"
+        None, "--path",
+        help="(required: publish, sync, publish-image, publish-version) Path to your snap code."
     ),
     resources_path: Union[str, None] = typer.Option(
-        None, "--resources-path", help="(optional: publish, sync, publish-image, publish-version; req: upload-docs) Path to resources such as your Dockerfile, snapser-byosnap-profile.json, snapser-tool-*.json, swagger.json or README.md"
+        None, "--resources-path",
+        help=(
+            "(required: upload-docs) (optional: publish, sync, publish-image, publish-version) "
+            "Path to resources such as your Dockerfile, snapser-byosnap-profile.json, "
+            "snapser-tool-*.json, swagger.json or README.md."
+        )
     ),
     # publish, sync and publish-version
     version: Union[str, None] = typer.Option(
         None, "--version",
-        help="(req: publish, sync, publish-version) Snap version. Should start with v. Example vX.X.X"
+        help=(
+            "(required: publish, sync, publish-version) Snap semantic version. "
+            "Should start with `v`. Example `vX.X.X`."
+        )
     ),
     # sync
     snapend_id: str = typer.Option(
         None, "--snapend-id",
-        help=("(req: sync) Snapend Id. NOTE: Development Snapends only.")
+        help=("(required: sync) Snapend Id. NOTE: Development Snapends only.")
     ),
     blocking: bool = typer.Option(
         False, "--blocking",
@@ -410,47 +447,64 @@ def byosnap(
     ),
     # create
     name: str = typer.Option(
-        None, "--name", help="(req: create) Name for your snap."
+        None, "--name", help="(required: create) Name for your snap."
     ),
     desc: str = typer.Option(
-        None, "--desc", help="(req: create) Description for your snap"
+        None, "--desc", help="(required: create) Description for your snap."
     ),
     platform_type: str = typer.Option(
         None, "--platform",
-        help="(req: create) Platform for your snap - " +
-        ", ".join(ByoSnap.PLATFORMS) + "."
+        help=(
+            "(required: create) Platform for your snap - " +
+            ", ".join(ByoSnap.PLATFORMS) + "."
+        )
     ),
     language: str = typer.Option(
         None, "--language",
-        help="(req: create) Language of your snap - " +
-        ", ".join(ByoSnap.LANGUAGES) + "."
+        help=(
+            "(required: create) Language of your snap - " +
+            ", ".join(ByoSnap.LANGUAGES) + "."
+        )
     ),
     # publish-image, publish-version, publish, sync, upload-docs
     tag: str = typer.Option(
-        None, "--tag", help=(
-            "(req: publish-image, publish-version, upload-docs) (optional: publish, sync) Tag for your snap"
+        None, "--tag",
+        help=(
+            "(required: publish-image, publish-version, upload-docs) (optional: publish, sync) "
+            "Tag for your snap."
         )
     ),
     # overrides
     skip_build: bool = typer.Option(
-        False, "--skip-build", help="(optional: publish-image, publish, sync) Skip the build step. You have to pass the image tag you used during the build step."
+        False, "--skip-build",
+        help=(
+            "(optional: publish-image, publish, sync) Skip the build step. You have to pass "
+            "the image tag you used during the build step."
+        )
     ),
     docker_filename: str = typer.Option(
-        "Dockerfile", help="(optional override: publish, sync) Dockerfile name to use"
+        "Dockerfile", help="(optional: publish, sync) Dockerfile name to use."
     ),
     profile_filename: str = typer.Option(
-        "snapser-byosnap-profile.json", "--profile-filename", help="(req: generate-profile, validate-profile) (optional override: publish, publish-version) BYOSnap Profile is picked up via the --path or the --resources-path. This allows you to override the default profile filename."
+        "snapser-byosnap-profile.json", "--profile-filename",
+        help=(
+            "(required: generate-profile, validate-profile) (optional override: publish, "
+            "publish-version) BYOSnap Profile is picked up via the --path or the "
+            "--resources-path. This allows you to override the default profile filename."
+        )
     ),
     out_path: Union[str, None] = typer.Option(
         None, "--out-path", help=(
-            "(req: generate-profile) Path to output the byosnap profile"
+            "(required: generate-profile) Path to output the byosnap profile."
         )
     ),
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="(optional override) API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="(optional override) Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -493,19 +547,23 @@ def game(
     # name
     name: str = typer.Option(
         None, "--name",
-        help=("(req: create) Name of your game: ")
+        help=("(required: create) Name of your game.")
     ),
     # overrides
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
-      Game commands
+      Game commands - DEPRECATED: Use Application commands instead
     """
+    warning(
+        "Game commands have been deprecated. Please use Application commands instead.")
     validate_command_context(ctx)
     game_obj: Game = Game(
         subcommand=subcommand,
@@ -518,6 +576,44 @@ def game(
     raise typer.Exit(code=SNAPCTL_SUCCESS)
 
 
+@app.command()
+@telemetry("application", subcommand_arg="subcommand")
+def application(
+    ctx: typer.Context,
+    # Required fields
+    subcommand: str = typer.Argument(
+        ..., help="Application Subcommands: " + ", ".join(Application.SUBCOMMANDS) + "."
+    ),
+    # name
+    name: str = typer.Option(
+        None, "--name",
+        help=("(required: create) Name of your application.")
+    ),
+    # overrides
+    api_key: Union[str, None] = typer.Option(
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
+    ),
+    profile: Union[str, None] = typer.Option(
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
+    ),
+) -> None:
+    """
+      Application commands
+    """
+    validate_command_context(ctx)
+    application_obj: Application = Application(
+        subcommand=subcommand,
+        base_url=ctx.obj['base_url'],
+        api_key=ctx.obj['api_key'],
+        name=name
+    )
+    getattr(application_obj, subcommand.replace('-', '_'))()
+    success(f"Application {subcommand} complete")
+    raise typer.Exit(code=SNAPCTL_SUCCESS)
+
+
 @app.command()
 @telemetry("generate", subcommand_arg="subcommand")
 def generate(
@@ -532,21 +628,24 @@ def generate(
     category: Union[str, None] = typer.Option(
         None, "--category",
         help=(
-            "(req: token) (token: " +
-              ", ".join(Generate.CATEGORIES['credentials']) + ')'
+            "(required: token) Supported category - " +
+              ", ".join(Generate.CATEGORIES['credentials']) + "."
         )
     ),
     out_path: Union[str, None] = typer.Option(
-        None, "--out-path", help=(
-            "(req: token) Path to output the byosnap profile"
+        None, "--out-path",
+        help=(
+            "(required: token) Path to output the byosnap profile."
         )
     ),
     # overrides
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -573,92 +672,105 @@ def snapend(
     subcommand: str = typer.Argument(
         ..., help="Snapend Subcommands: " + ", ".join(Snapend.SUBCOMMANDS) + "."
     ),
-    # snapend_id: str = typer.Argument(..., help="Snapend Id"),
     snapend_id: str = typer.Option(
         None, "--snapend-id",
-        help=("(req: state, update, download) Snapend Id")
+        help=("(required: state, update, download) Snapend Id.")
     ),
     # enumerate
     game_id: str = typer.Option(
         None, "--game-id",
-        help="(req: enumerate, clone) Game Id"
+        help="(DEPRECATED: Use --application-id instead) Game Id."
+    ),
+    application_id: str = typer.Option(
+        None, "--application-id",
+        help="(required: enumerate, create, clone) Application Id."
     ),
     # apply, clone
     manifest_path_filename: str = typer.Option(
         None, "--manifest-path-filename",
-        help="(req: apply|clone) Full Path to the manifest file including the filename."
+        help=(
+            "(required: create, apply, clone) Full Path to the manifest file "
+            "including the filename."
+        )
     ),
     force: bool = typer.Option(
         False, "--force",
-        help="(optional: apply) If true, Snapser will ignore the configuration diff validation and allow to force apply the manifest."
+        help=(
+            "(optional: apply) If true, Snapser will ignore the configuration diff validation "
+            "and allow to force apply the manifest."
+        )
     ),
     # download
     category: str = typer.Option(
         None, "--category",
         help=(
-            "(req: download) Category of the Download: " +
+            "(required: download) Supported Download Categories - " +
             ", ".join(Snapend.DOWNLOAD_CATEGORY) + "."
         )
     ),
     category_format: str = typer.Option(
         None, "--format",
         help=(
-            "(req: --category sdk|protos|snapend-manifest --format  "
+            "(required: --category sdk|protos|snapend-manifest --format  " +
             "sdk(" + ", ".join(SDK_TYPES.keys()) +
             ") | protos(" + ", ".join(PROTOS_TYPES.keys()) + ")" +
             ") | snapend-manifest(" +
-            ", ".join(SNAPEND_MANIFEST_TYPES.keys()) + ")"
+            ", ".join(SNAPEND_MANIFEST_TYPES.keys()) + ")."
         )
     ),
     category_type: str = typer.Option(
         None, "--type",
         help=(
-            "(optional: download) Only applicable for --category sdk|protos --type "
-            "sdk(" + ", ".join(Snapend.CATEGORY_TYPE_SDK) + ")"
-            " | protos(" + ", ".join(Snapend.CATEGORY_TYPE_PROTOS) + ")"
+            "(optional: download) Only applicable for --category sdk|protos --type " +
+            "sdk(" + ", ".join(Snapend.CATEGORY_TYPE_SDK) + ")" +
+            " | protos(" + ", ".join(Snapend.CATEGORY_TYPE_PROTOS) + ")."
         )
     ),
     category_http_lib: str = typer.Option(
         None, "--http-lib",
         help=(
-            "(optional: download) Only applicable for --category sdk "
+            "(optional: download) Only applicable for --category sdk " +
             "--format " + '|'.join(Snapend.get_formats_supporting_http_lib()) + ' ' +
-            "--type user|server|internal|app "
-            "--http-lib " + Snapend.get_http_formats_str()
+            "--type user|server|internal|app " +
+            "--http-lib " + Snapend.get_http_formats_str() + "."
         )
     ),
-    snaps: Union[str, None] = typer.Option(
+    snaps_list_str: Union[str, None] = typer.Option(
         None, "--snaps",
         help=(
-            "(optional: download) Comma separated list of snap ids to customize the "
-              "SDKs, protos or admin settings. "
-              "snaps(" + ", ".join(SERVICE_IDS)
+            "(optional: download) Comma separated list of snap ids to customize the " +
+              "SDKs, protos or admin settings. " +
+              "snaps(" + ", ".join(SERVICE_IDS) + ")."
         )
     ),
     # Clone
     name: Union[str, None] = typer.Option(
-        None, "--name", help="(req: clone) Snapend name"),
+        None, "--name",
+        help="(required: clone, optional: create) Snapend name."),
     env: Union[str, None] = typer.Option(
-        None, "--env", help=(
-            "(req: clone) Snapend environment"
-            "Environments: (" + ", ".join(Snapend.ENV_TYPES) + ")"
+        None, "--env",
+        help=(
+            "(required: clone, optional: create) Snapend environments " +
+            " - " + ", ".join(Snapend.ENV_TYPES) + "."
         )),
     # Download, Apply, Clone
     out_path: Union[str, None] = typer.Option(
-        None, "--out-path", help="(optional: download|apply|clone) Path to save the output file"),
+        None, "--out-path",
+        help="(optional: create, download, apply, clone) Path to save the output file."
+    ),
     # update
     byosnaps_list: str = typer.Option(
         None, "--byosnaps",
         help=(
             "(optional: update) Comma separated list of BYOSnap ids and versions. "
-            "Eg: service-1:v1.0.0,service-2:v1.0.0"
+            "Eg: `service-1:v1.0.0,service-2:v1.0.0`."
         )
     ),
     byogs_list: str = typer.Option(
         None, "--byogs",
         help=(
             "(optional: update) Comma separated list of BYOGs fleet_name:tag. "
-            "Eg: fleet-1:v1.0.0,fleet-2:v1.0.0"
+            "Eg: `fleet-1:v1.0.0,fleet-2:v1.0.0`."
         )
     ),
     # create, update, promote, apply, clone
@@ -671,10 +783,12 @@ def snapend(
     ),
     # overrides
     api_key: Union[str, None] = typer.Option(
-        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -687,7 +801,7 @@ def snapend(
         api_key=ctx.obj['api_key'],
         snapend_id=snapend_id,
         # Enumerate, Clone
-        game_id=game_id,
+        game_id=application_id if application_id is not None else game_id,
         # Clone
         name=name, env=env,
         # Apply, Clone
@@ -698,7 +812,7 @@ def snapend(
         category_format=category_format,
         category_type=category_type,
         category_http_lib=category_http_lib,
-        snaps=snaps,
+        snaps=snaps_list_str,
         # Download, Apply and Clone
         out_path=out_path,
         # Update
@@ -720,26 +834,33 @@ def byows(
     # attach
     snapend_id: str = typer.Option(
         None, "--snapend-id",
-        help=("(req: attach, reset) Your Snapend Id")
+        help=("(required: attach, reset) Snapend Id.")
     ),
     byosnap_id: str = typer.Option(
         None, "--byosnap-id",
-        help=("(req: attach) Your BYOSnap Id")
+        help=("(required: attach) BYOSnap Id. Should start with `byosnap-`.")
     ),
     http_port: str = typer.Option(
         None, "--http-port",
-        help=("(oneof: attach) HTTP port of your local server. One of --http-port or --grpc-port is required.")
+        help=(
+            "(use: attach) HTTP port of your local server. "
+            "One of --http-port or --grpc-port is required."
+        )
     ),
     grpc_port: str = typer.Option(
         None, "--grpc-port",
-        help=("(oneof: attach) gRPC port of your local server. One of --http-port or --grpc-port is required.")
+        help=(
+            "(use: attach) gRPC port of your local server. "
+            "One of --http-port or --grpc-port is required."
+        )
     ),
     # overrides
     api_key: Union[str, None] = typer.Option(
         None, "--api-key", help="API Key override.", callback=api_key_context_callback
     ),
     profile: Union[str, None] = typer.Option(
-        None, "--profile", help="Profile from the Snapser config to use.", callback=profile_context_callback
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
     ),
 ) -> None:
     """
@@ -759,3 +880,152 @@ def byows(
     getattr(byows_obj, subcommand.replace('-', '_'))()
     success(f"BYOWs {subcommand} complete")
     raise typer.Exit(code=SNAPCTL_SUCCESS)
+
+
+@app.command()
+@telemetry("snaps", subcommand_arg="subcommand")
+def snaps(
+    ctx: typer.Context,
+    # Required fields
+    subcommand: str = typer.Argument(
+        ..., help="Snaps Subcommands: " + ", ".join(Snaps.SUBCOMMANDS) + "."
+    ),
+    out_path_filename: Union[str, None] = typer.Option(
+        None, "--out-path-filename",
+        help=(
+            "(optional: enumerate) Path and filename to output the snaps list. "
+            "The filename should end with `.json`."
+        )
+    ),
+    # overrides
+    api_key: Union[str, None] = typer.Option(
+        None, "--api-key", help="API Key override.", callback=api_key_context_callback
+    ),
+    profile: Union[str, None] = typer.Option(
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
+    ),
+) -> None:
+    """
+      Bring your own workstation commands
+    """
+    validate_command_context(ctx)
+    snaps_obj: Snaps = Snaps(
+        subcommand=subcommand,
+        base_url=ctx.obj['base_url'],
+        api_key=ctx.obj['api_key'],
+        out_path_filename=out_path_filename,
+    )
+    getattr(snaps_obj, subcommand.replace('-', '_'))()
+    success(f"Snaps {subcommand} complete")
+    raise typer.Exit(code=SNAPCTL_SUCCESS)
+
+
+@app.command()
+@telemetry("snapend_manifest", subcommand_arg="subcommand")
+def snapend_manifest(
+    ctx: typer.Context,
+    # Required fields
+    subcommand: str = typer.Argument(
+        ..., help="Snapend Manifest Subcommands: " + ", ".join(SnapendManifest.SUBCOMMANDS) + "."
+    ),
+    name: Union[str, None] = typer.Option(
+        None, "--name",
+        help="(required: create) Name for your snapend."
+    ),
+    env: Union[str, None] = typer.Option(
+        None, "--env",
+        help=(
+            "(required: create) Environment for your snapend - " +
+            ", ".join(SnapendManifest.ENVIRONMENTS) + "."
+        )
+    ),
+    manifest_path_filename: Union[str, None] = typer.Option(
+        None, "--manifest-path-filename",
+        help=(
+            "(required: sync, upgrade, update) Full Path to the manifest file "
+            "including the filename."
+        )
+    ),
+    snaps_list_str: str = typer.Option(
+        None, "--snaps",
+        help=(
+            "(use: create, sync, upgrade) Comma separated list of snap ids to add, " +
+            "sync or upgrade. snaps(" + ", ".join(SERVICE_IDS) + ")."
+        )
+    ),
+    features: str = typer.Option(
+        None, "--features",
+        help=(
+            "(use: create, sync) Comma separated list of feature flags to add, sync. " +
+            "features(" + ", ".join(SnapendManifest.FEATURES) + ")."
+        )
+    ),
+    add_snaps: str = typer.Option(
+        None, "--add-snaps",
+        help=(
+            "(use: update) Comma separated list of snap ids to add. " +
+            "snaps(" + ", ".join(SERVICE_IDS) + ")."
+        )
+    ),
+    remove_snaps: str = typer.Option(
+        None, "--remove-snaps",
+        help=(
+            "(use: update) Comma separated list of snap ids to remove. snaps(" +
+            ", ".join([s for s in SERVICE_IDS if s.lower() != SnapendManifest.AUTH_SNAP_ID]) +
+            ")."
+        )
+    ),
+    add_features: str = typer.Option(
+        None, "--add-features",
+        help=(
+            "(use: update) Comma separated list of features to add. " +
+            "features(" + ", ".join(SnapendManifest.FEATURES) + ")."
+        )
+    ),
+    remove_features: str = typer.Option(
+        None, "--remove-features",
+        help=(
+            "(use: update) Comma separated list of features to remove. " +
+            "features(" + ", ".join(SnapendManifest.FEATURES) + ")."
+        )
+    ),
+    out_path_filename: Union[str, None] = typer.Option(
+        None, "--out-path-filename",
+        help=(
+            "(required: create, sync, upgrade, update) (optional: enumerate) Path and filename to "
+            "output the manifest. The filename should end with .json or .yaml"
+        )
+    ),
+    # overrides
+    api_key: Union[str, None] = typer.Option(
+        None, "--api-key",
+        help="API Key override.", callback=api_key_context_callback
+    ),
+    profile: Union[str, None] = typer.Option(
+        None, "--profile",
+        help="Profile from the Snapser config to use.", callback=profile_context_callback
+    ),
+) -> None:
+    """
+      Bring your own workstation commands
+    """
+    validate_command_context(ctx)
+    snapend_manifest_obj: SnapendManifest = SnapendManifest(
+        subcommand=subcommand,
+        base_url=ctx.obj['base_url'],
+        api_key=ctx.obj['api_key'],
+        name=name,
+        environment=env,
+        manifest_path_filename=manifest_path_filename,
+        snaps=snaps_list_str,
+        features=features,
+        add_snaps=add_snaps,
+        remove_snaps=remove_snaps,
+        add_features=add_features,
+        remove_features=remove_features,
+        out_path_filename=out_path_filename,
+    )
+    getattr(snapend_manifest_obj, subcommand.replace('-', '_'))()
+    success(f"Snapend Manifest {subcommand} complete")
+    raise typer.Exit(code=SNAPCTL_SUCCESS)
